; TEA assembly implementation (x) herm1t@vx.netlux.org 2006-08-20
;
;	void encrypt(unsigned long* v, unsigned long* k);
;	void decrypt(unsigned long* v, unsigned long* k);
;
;    TEA operates on 64-bit blocks and uses a 128-bit key. It has a Feistel
; structure with a suggested 64 rounds, typically implemented in pairs termed
; cycles. It has an extremely simple key schedule, mixing all of the key
; material in exactly the same way for each cycle. Different multiples of a
; magic constant are used to prevent simple attacks based on the symmetry of
; the rounds.
;    TEA has a few weaknesses. Most notably, it suffers from equivalent keys
; each key is equivalent to three others, and this means that the effective
; key size is only 126 bits (Kelsey et. al., 1996, and Vikram Andem, 2003).
; This weakness led to a method for hacking Microsoft's Xbox game console,
; where the cipher was used as a hash function. TEA is also susceptible to a
; related-key attack which requires 223 chosen plaintexts under a related-key
; pair, with 232 time complexity (Kelsey et. al., 1997).
		BITS	32
		CPU	386
		global	encrypt,decrypt
%macro	tea	2
	mov	ecx, %1			;v1/v0
	shl	ecx, 4
	add	ecx, [edi + %2 + 0]	;k0/k2
	mov	edx, %1			;v1/v0
	add	edx, ebp		;sum
	xor	ecx, edx
	mov	edx, %1			;v1/v0
	shr	edx, 5
	add	edx, [edi + %2 + 4]	;k1/k3
	xor	ecx, edx
%endmacro
%macro	init	0
	pusha
	mov	esi, [esp + 36]		; data
	mov	edi, [esp + 40]		; key
	mov	eax, [esi + 0]		; v0
	mov	ebx, [esi + 4]		; v1
	push	byte 32
	pop	ecx
%endmacro
%macro	fini	0
	pop	ecx	
	loop	.l0
	mov	[esi + 0], eax
	mov	[esi + 4], ebx
	popa
	ret
%endmacro
%define	DELTA	0x9e3779b9
encrypt:	init
		xor	ebp, ebp
	.l0:	push	ecx
		add	ebp, DELTA
		tea	ebx, 0
		add	eax, ecx
		tea	eax, 8
		add	ebx, ecx
		fini
decrypt:	init
		mov	ebp, 0xc6ef3720
	.l0:	push	ecx
		tea	eax, 8
		sub	ebx, ecx
		tea	ebx, 0
		sub	eax, ecx
		sub	ebp, DELTA
		fini
;EOF
